package org.netkernel.xml.saxon.accessor.serialize;

/******************************************************************************
  (c) Copyright 20011, 1060 Research Ltd

  This Software is licensed to You, the licensee, for use under the terms of
  the 1060 Public License v1.0. Please read and agree to the 1060 Public
  License v1.0 [www.1060research.com/license] before using or redistributing
  this software.

  In summary the 1060 Public license has the following conditions.
  A. You may use the Software free of charge provided you agree to the terms
  laid out in the 1060 Public License v1.0
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.
  You may write your own software for execution by this Software provided any
  distribution of your software with this Software complies with terms set out
  in section 2 of the 1060 Public License v1.0
  C. You may redistribute the Software provided you comply with the terms of
  the 1060 Public License v1.0 and that no warranty is implied or given.
  D. If you find you are unable to comply with this license you may seek to
  obtain an alternative license from 1060 Research Limited by contacting
  license@1060research.com or by visiting www.1060research.com

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
  LICENSE V1.0 FOR DETAILS

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license

  $Id$
 *****************************************************************************/

import net.sf.saxon.s9api.Serializer;
import net.sf.saxon.s9api.XdmNode;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.util.XMLReadable;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.saxon.util.SaxonUtil;
import org.w3c.dom.Document;

import java.io.ByteArrayOutputStream;

/**
 * Serializes an XdmNode
 *
 * @author Chris Cormack
 */
public class SaxonSerializerAccessor extends StandardAccessorImpl {
  public SaxonSerializerAccessor() {
    this.declareThreadSafe();
  }

  @Override
  public void onSource(INKFRequestContext aContext) throws Exception {
    XdmNode operand = aContext.source("arg:operand", XdmNode.class);

    String mimeType = null;

    Serializer serializer = new Serializer();
    ByteArrayOutputStream baos = new ByteArrayOutputStream(2048);
    serializer.setOutputStream(baos);

    if (aContext.exists("arg:operator")) {
      IHDSNode serializeConfig = aContext.source("arg:operator", IHDSNode.class);

      String encoding = (String)serializeConfig.getFirstValue("/serialize/encoding");
      String indent = (String)serializeConfig.getFirstValue("/serialize/indent");
      String omitDeclaration = (String)serializeConfig.getFirstValue("/serialize/omit-declaration");
      String doctypePublic = (String)serializeConfig.getFirstValue("/serialize/doctype/public");
      String doctypeSystem = (String)serializeConfig.getFirstValue("/serialize/doctype/system");
      String method = (String)serializeConfig.getFirstValue("/serialize/method");
      mimeType = (String)serializeConfig.getFirstValue("/serialize/mimeType");

      if (encoding != null) {
        serializer.setOutputProperty(Serializer.Property.ENCODING, encoding);
      }
      if (indent != null) {
        serializer.setOutputProperty(Serializer.Property.INDENT, indent);
      }
      if (omitDeclaration != null) {
        serializer.setOutputProperty(Serializer.Property.OMIT_XML_DECLARATION, omitDeclaration);
      }
      if (doctypePublic != null) {
        serializer.setOutputProperty(Serializer.Property.DOCTYPE_PUBLIC, doctypePublic);
      }
      if (doctypeSystem != null) {
        serializer.setOutputProperty(Serializer.Property.DOCTYPE_SYSTEM, doctypeSystem);
      }
      if (method == null) {
        method= "xml";
      }
      serializer.setOutputProperty(Serializer.Property.METHOD, method);
    }

    SaxonUtil.getProcessor().writeXdmValue(operand, serializer);

    INKFResponse resp = aContext.createResponseFrom(new ByteArrayRepresentation(baos));

    if (mimeType == null) {
      mimeType = "application/xml";
    }

    resp.setMimeType(mimeType);
  }
}
