/******************************************************************************
  (c) Copyright 2009, 1060 Research Ltd                                        

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  $Id$
 *****************************************************************************/
package org.netkernel.xml.saxon.accessor.xslt2;

import java.math.BigDecimal;
import java.net.URI;
import java.util.Map;

import net.sf.saxon.s9api.QName;
import net.sf.saxon.s9api.XdmAtomicValue;
import net.sf.saxon.s9api.XdmDestination;
import net.sf.saxon.s9api.XdmNode;
import net.sf.saxon.s9api.XdmValue;
import net.sf.saxon.s9api.XsltExecutable;
import net.sf.saxon.s9api.XsltTransformer;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.saxon.util.LocalURIResolver;
import org.netkernel.xml.saxon.util.NKCollectionURIResolver;
import org.netkernel.xml.saxon.util.TypeMap;

import javax.xml.transform.OutputKeys;

/**
 * Provides a NetKernel 4 accessor to XSLT2 using Saxon9.
 * 
 * See <a href="/book/view/book:xml:saxon:book/doc:xml:saxon:xslt2">doc:xml:saxon:xslt2</a>.
 * 
 * @author Chris Cormack
 * @author Menzo Windhouwer
 */
public class XSLTAccessor extends StandardAccessorImpl
{
	public XSLTAccessor()
	{	this.declareThreadSafe();
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	XsltTransformer transformer = context.source("arg:operator",
				XsltExecutable.class).load();
		XdmNode source = context.source("arg:operand", XdmNode.class);

		Map<String, Class> typeMap = null;
		if (context.exists("arg:typemap"))
		{	typeMap = context.source("arg:typemap", TypeMap.class).getMap();
		}

		for (int i = 0; i < context.getThisRequest().getArgumentCount(); i++)
		{	String argName = context.getThisRequest().getArgumentName(i);
			if (!(argName.equals("operand")
					|| argName.equals("operator")
					|| argName.equals("scheme")
					|| argName.equals("activeType")
					|| argName.equals("typemap")))
			{	Object argValue = context.source(context.getThisRequest()
						.getArgumentValue(i));

				if (typeMap != null && typeMap.containsKey(argName))
				{	argValue = context.source(context.getThisRequest()
							.getArgumentValue(i), typeMap.get(argName));
				} else
				{	argValue = context.source(context.getThisRequest()
							.getArgumentValue(i));
				}

				XdmValue value = null;
				if (argValue == null)
				{

				}
				if (argValue instanceof String)
				{	value = new XdmAtomicValue((String) argValue);
				} else if (argValue instanceof Float)
				{	value = new XdmAtomicValue((Float) argValue);
				} else if (argValue instanceof Integer)
				{	value = new XdmAtomicValue((Integer) argValue);
				} else if (argValue instanceof Boolean)
				{	value = new XdmAtomicValue((Boolean) argValue);
				} else if (argValue instanceof Long)
				{	value = new XdmAtomicValue((Long) argValue);
				} else if (argValue instanceof URI)
				{	value = new XdmAtomicValue((URI) argValue);
				} else if (argValue instanceof BigDecimal)
				{	value = new XdmAtomicValue((BigDecimal) argValue);
				} else
				{	value = context.source(context.getThisRequest()
							.getArgumentValue(i), XdmNode.class);
				}

				transformer.setParameter(new QName(argName), value);
			}
		}

		synchronized (transformer.getUnderlyingController().getConfiguration())
		{	XdmDestination result = new XdmDestination();

			LocalURIResolver resolver = new LocalURIResolver(context);
			transformer.getUnderlyingController().setUnparsedTextURIResolver(
					resolver);
			transformer.getUnderlyingController().setURIResolver(
					resolver);
			transformer.getUnderlyingController().getConfiguration()
					.setCollectionURIResolver(new NKCollectionURIResolver(context));

			transformer.setInitialContextNode(source);
			transformer.setDestination(result);
			transformer.transform();

      INKFResponse resp= context.createResponseFrom(result.getXdmNode());

      String mimeType= transformer.getUnderlyingController().getOutputProperty(OutputKeys.MEDIA_TYPE);
      if (mimeType != null) {
        resp.setMimeType(mimeType);
      }
		}
	}
}
